// Copyright © Advanced Micro Devices, Inc., or its affiliates.
// SPDX-License-Identifier:  MIT

#pragma once

// NOLINTBEGIN
/*!
 * @brief Maximum error message size (including the null terminator).
 *
 * This value is used across the API and backend to size buffers for error strings.
 */
#define HIPDNN_ERROR_STRING_MAX_LENGTH 2048
// NOLINTEND
