// Copyright © Advanced Micro Devices, Inc., or its affiliates.
// SPDX-License-Identifier: MIT

#pragma once

// Include all pointwise operation functors organized by arity
#include <hipdnn_sdk/test_utilities/pointwise/BinaryOperationFunctors.hpp>
#include <hipdnn_sdk/test_utilities/pointwise/UnaryOperationFunctors.hpp>
