// Copyright © Advanced Micro Devices, Inc., or its affiliates.
// SPDX-License-Identifier: MIT

#pragma once

namespace hipdnn_sdk::utilities
{
constexpr double BATCHNORM_DEFAULT_EPSILON = 1e-5;
} // namespace hipdnn_sdk::utilities
