// Copyright © Advanced Micro Devices, Inc., or its affiliates.
// SPDX-License-Identifier: MIT

#pragma once

typedef enum
{
    HIPDNN_HEUR_MODE_FALLBACK = 0, // Functional support only, no performance guarantees
} hipdnnBackendHeurMode_t;
