// automatically generated by the FlatBuffers compiler, do not modify


#ifndef FLATBUFFERS_GENERATED_DATATYPES_HIPDNN_SDK_DATA_OBJECTS_H_
#define FLATBUFFERS_GENERATED_DATATYPES_HIPDNN_SDK_DATA_OBJECTS_H_

#include "flatbuffers/flatbuffers.h"

// Ensure the included flatbuffers.h is the same version as when this file was
// generated, otherwise it may not be compatible.
static_assert(FLATBUFFERS_VERSION_MAJOR == 25 &&
              FLATBUFFERS_VERSION_MINOR == 12 &&
              FLATBUFFERS_VERSION_REVISION == 19,
             "Non-compatible flatbuffers version included");


namespace hipdnn_sdk {
namespace data_objects {

enum class DataType : int8_t {
  UNSET = 0,
  FLOAT = 1,
  HALF = 2,
  BFLOAT16 = 3,
  DOUBLE = 4,
  UINT8 = 5,
  INT32 = 6,
  MIN = UNSET,
  MAX = INT32
};

inline const DataType (&EnumValuesDataType())[7] {
  static const DataType values[] = {
    DataType::UNSET,
    DataType::FLOAT,
    DataType::HALF,
    DataType::BFLOAT16,
    DataType::DOUBLE,
    DataType::UINT8,
    DataType::INT32
  };
  return values;
}

inline const char * const *EnumNamesDataType() {
  static const char * const names[8] = {
    "UNSET",
    "FLOAT",
    "HALF",
    "BFLOAT16",
    "DOUBLE",
    "UINT8",
    "INT32",
    nullptr
  };
  return names;
}

inline const char *EnumNameDataType(DataType e) {
  if (::flatbuffers::IsOutRange(e, DataType::UNSET, DataType::INT32)) return "";
  const size_t index = static_cast<size_t>(e);
  return EnumNamesDataType()[index];
}

}  // namespace data_objects
}  // namespace hipdnn_sdk

#endif  // FLATBUFFERS_GENERATED_DATATYPES_HIPDNN_SDK_DATA_OBJECTS_H_
